@@ -1,7 +1,6 @@
 context('Test profile analysis')
 
 library(dplyr)
-library(tibble)
 library(DBI)
 library(RSQLite)
 
@@ -17,7 +16,7 @@ verbAggCopy = function(pth = '../verbAggression.db')
 }
 
 
-
+# to do: check for proper number of rows
 
 test_that('profile analysis verb agg',{
   db = verbAggCopy()
@@ -28,12 +27,12 @@ test_that('profile analysis verb agg',{
   expect_gt(cor(p$domain_score,p$expected_domain_score), 0.6, 
             'expected score should have a relation with observed score')
   
-  expect_gt(cor(p$domain_score,p$expected_domain_score), cor(p$sumScore,p$expected_domain_score),
+  expect_gt(cor(p$domain_score,p$expected_domain_score), cor(p$booklet_score,p$expected_domain_score),
             'domain should add extra information')
   
   expect_true(all(p %>% 
                     group_by(person_id) %>% 
-                    summarise(sum_dif = abs(sum(expected_domain_score) - first(sumScore))) %>%
+                    summarise(sum_dif = abs(sum(expected_domain_score) - first(booklet_score))) %>%
                     ungroup() %>%
                     pull(sum_dif) < 1e-10), 
               'expected domains scores need to sum to total test score')
@@ -41,20 +40,22 @@ test_that('profile analysis verb agg',{
   # check inputs work with just parms
   pt = profile_tables(f, get_items(db),'situation')
   
-  # takes a little long for cran
-  skip_on_cran()
+
+  # to do: test especially profiles bayes against old dexter
+
   f = fit_enorm(db, method='Bayes')
   p = profiles(db, f, 'behavior')
   
   expect_gt(cor(p$domain_score,p$expected_domain_score), 0.6, 
             'expected score should have a relation with observed score (Bayes)')
   
-  expect_gt(cor(p$domain_score,p$expected_domain_score), cor(p$sumScore,p$expected_domain_score),
+  
+  expect_gt(cor(p$domain_score,p$expected_domain_score), cor(p$booklet_score,p$expected_domain_score),
             'domain should add extra information (Bayes)')
   
   expect_true(all(p %>% 
                     group_by(person_id) %>% 
-                    summarise(sum_dif = abs(sum(expected_domain_score) - first(sumScore))) %>%
+                    summarise(sum_dif = abs(sum(expected_domain_score) - first(booklet_score))) %>%
                     ungroup() %>%
                     pull(sum_dif) < 1e-10), 
               'expected domains scores need to sum to total test score (Bayes)')
